@@ -15,6 +15,7 @@
 
 import datetime
 import io
+import sys
 import os
 
 import invoke
@@ -139,7 +140,19 @@ def reload_uwsgi(c):
     # you might `source`. It has "env = " at the start instead of export,
     # doesn't mind whitespace in the values and treats quotes literally.
     #
-    # See here on getting quotes right https://lists.gnu.org/archive/html/fab-user/2013-01/msg00005.html.
+    # See here on getting quotes right in Fabric
+    # https://lists.gnu.org/archive/html/fab-user/2013-01/msg00005.html.
+
+    # Don't use percent characters in environment variables because uWSGI will
+    # silently drop them unless they are doubled-up (meaning that you'll have a
+    # hard time tracking down the subtle bug).
+    try:
+        c.run("! grep '%' {env.project_dir}/env".format(env=c.env))
+    except invoke.exceptions.UnexpectedExit as e:
+        print('Environment variables should not contain "%" due to its special use in uWSGI config.', file=sys.stderr)
+        raise
+
+    # Removes quotes as these are interpreted literally by uWSGI.
     c.sudo('echo "" >> /etc/uwsgi-emperor/vassals/{env.site_name}.ini'.format(env=c.env))
     c.sudo("""sed 's/export/env =/' {env.project_dir}/env | sed "s/['\\"]//g" >> /etc/uwsgi-emperor/vassals/{env.site_name}.ini""".format(env=c.env))
 
@@ -201,7 +214,17 @@ def mirror_media(c):
 def lint(c):
     """Run Pylint over everything."""
 
-    c.local('python3 -m pylint --rcfile=pylint.conf $(find -path ./node_modules -prune -o -type f -name \'*.py\' -print)')
+    # --jobs=0 enable parallelism based on number of cores available.
+    c.local("git ls-files '*.py' | xargs python3 -m pylint --jobs=0 --rcfile=pylint.conf")
+
+
+def flake8_test(c):
+    # See .flake8 for excluded checks.
+    c.local("git ls-files '*.py' | xargs python3 -m flake8")
+
+
+def mypy_test(c):
+    c.local("git ls-files '*.py' | xargs python3 -m mypy --config-file mypy.ini")
 
 
 def grep_for_pdb(c, exclude=None):
